#include "datapreprocessor.h"

#include <QDebug>
#include <qmath.h>

DataPreProcessor::DataPreProcessor()
{
    this->areaCount = 0;
    this->areas = new area_t*();
}

/**
 * @brief Funkce pro přidávání nových měření.
 *
 * V této funkci jsou počítány souřadnice x, y a také distanční koeficient pro všechny body. Pole oblastí (area_t) se automaticky realokuje.
 * @param data - Jedno měření (otáčka skeneru) uložená do area_t.
 */
void DataPreProcessor::addNewArea(data_t &data)
{
    int index = this->relocateAreaArray();
    areas[index]->pointCount = data.measureCount;
    areas[index]->points = new QPointF[data.measureCount];
    areas[index]->distanceCoef = new qreal[data.measureCount];

    qreal qualityThis = 0;
    int realCount = 0;

    for(int i = 0, h = 0; i < data.measureCount; i++, h++) {
        if(data.measure[i].quality > 3 && data.measure[i].distance > 15 && data.measure[i].distance < 7000) {
            qreal x = data.measure[i].distance * qCos(data.measure[i].angle * 3.14 / 180);
            qreal y = data.measure[i].distance * qSin(data.measure[i].angle * 3.14 / 180);

            areas[index]->points[h].setX(x);
            areas[index]->points[h].setY(y);
            areas[index]->distanceCoef[h] = 0.224*data.measure[i].distance - 0.0333;

            qualityThis += data.measure[i].quality;
            realCount++;
        } else {
            h--;
            areas[index]->pointCount -= 1;
        }
    }

    qualityThis /= realCount;
    this->quality = (this->quality + qualityThis)/2;
}

/**
 * @brief Funkce pro realokaci pole oblastí area_t, je volána automaticky.
 * @return Aktuální používaný index pole area_t.
 */
int DataPreProcessor::relocateAreaArray()
{
    this->oldAreas = this->areas;
    this->areas = new area_t*[this->areaCount + 1];
    for(int i = 0; i < this->areaCount + 1; i++) {
        this->areas[i] = new area_t();
    }
    for(int i = 0; i < this->areaCount; i++) {
        this->areas[i] = this->oldAreas[i];
    }
    this->areas[this->areaCount]->pointCount = 0;
    this->areaCount++;
    this->oldAreas = NULL;
    return (this->areaCount - 1);
}
