#ifndef DATAPREPROCESSOR_H
#define DATAPREPROCESSOR_H

#include <QObject>
#include <QVector>

#include "datastructures.h"

/**
 * @brief Třída pro ukládání měření do oblastí area_t bůď ze skeneru nebo z načtených souborů.
 *
 * Zde jsou uloženy všechny oblasti. Objekt není závislý na počtu oblastí, protože si jejich počet hlídá a automaticky realokuje pole kde jsou uloženy. Realokace i hlídání velikosti pole je automatické. Oblasti jsou uloženy v **pointer (směrník) poli.
 */
class DataPreProcessor : public QObject
{
    Q_OBJECT

private slots:
    int relocateAreaArray();

public slots:
    void addNewArea(data_t &data);

public:
    DataPreProcessor();
    area_t **areas; /**< Pole oblastí reprezentováno jako dvou * směrník. */
    int areaCount; /**< Počet všech oblastí. */
    int quality; /**< Průměrná kvalita měření. */

private:
    area_t **oldAreas;

};

#endif // DATAPREPROCESSOR_H
