#include "datareader.h"

#include <QTextStream>
#include <QDebug>

/**
 * @brief Konstruktor, který v prvním kroce spočítá počet měření uložených v souboru a podle toho alokuje a naplní pole data_t.
 * @param path - Cesta k souboru (musí být ve stejném adresáři jako aplikace).
 * @param parent
 */
DataReader::DataReader(QString path, QObject *parent) : QObject(parent)
{
    this->dataCount = 0;
    this->index = 0;
    this->measureCount = QVector<int>(1, 0);
    this->file = new QFile(path);
    if (!file->open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "error";
    }
    QTextStream in(file);
    QString line = in.readLine();
    while(!in.atEnd()) {
        this->countLine(line);
        line = in.readLine();
    }
    this->data = new data_t[this->dataCount];
    for(int i = 0; i < this->dataCount; i++) {
        this->data[i].measureCount = this->measureCount.at(i);
        this->data[i].measure = new measure_t[this->measureCount[i]];
    }
    file->reset();
    QString preLine = NULL;
    line = in.readLine();
    index = 0;
    measureIndex = 0;
    while(!in.atEnd()) {
        this->readLine(preLine, line);
        preLine = line;
        line = in.readLine();
    }
    file->close();
}

/**
 * @brief Počítá kolik měření je celkem k alokaci data_t a také počítá kolik vzorků je v každém měření.
 * @param line - Jeden řádek ze souboru.
 */
void DataReader::countLine(QString line)
{
    int endCount = line.count("-END-");
    this->dataCount += endCount;
    this->measureCount[index] += line.count("|");
    if(endCount > 0) {
        this->measureCount.push_back(0);
        index++;
    }
}

/**
 * @brief Naparsuje jeden řádek ze souboru a uloží naměřené hodnoty.
 *
 * Zde je požadován i předchozí řádek zdůvodu toho, že se může stát, že jeden vzorek je rozdělen do dvou řádků.
 *
 * @param preLine - Řádek z předchozí iterace.
 * @param line - Aktuální řádek s hodnotami.
 */
void DataReader::readLine(QString preLine, QString line)
{
    if(line.count("Time") == 0) {
        int endCount = line.count("-END-");
        QStringList preData = preLine.split("|");
        QStringList data = line.split("|");
        QString start = data[0];
        if(start.count("___") == 0 && start.count("S__") == 0) {
            // start in preLine
            QString end = preData[preData.size()-1];
            end = end + start;
            this->saveDateFromString(end, measureIndex-1);
        }
        for(int i = 0; i < data.size(); i++) {
            this->saveDateFromString(data[i], measureIndex);
            measureIndex++;
        }
        if(endCount > 0) {
            index++;
            measureIndex = 0;
        }
    }
}

/**
 * @brief Uloží naměřené hodnoty z QString.
 * @param str - Jeden vzorek, který obsahuje měření vzdálenosti, úhlu a kvality.
 * @param m - Index vzorku v aktuálním měření.
 */
void DataReader::saveDateFromString(QString str, int m)
{
    QStringList list = str.split("_");
    for(int i = 3; i < list.size(); i++) {
        QString value = list[i];
        if(value.count("t") > 0) {
            value.replace("t:", "");
            data[index].measure[m].angle = value.toFloat();
        } else if(value.count("d") > 0) {
            value.replace("d:", "");
            data[index].measure[m].distance = value.toFloat();
        } else if(value.count("q") > 0) {
            value.replace("q:", "");
            data[index].measure[m].quality = value.toInt();
        }
    }
}

