#ifndef LANDMARKEXTRACTION_H
#define LANDMARKEXTRACTION_H

#include <QObject>
#include <QPointF>

#include "datastructures.h"

/**
 * @brief Třída hledající důležité objekty, hrany nebo rohy.
 */
class LandmarkExtraction : public QObject
{
    Q_OBJECT

public:
    LandmarkExtraction();

public slots:
    void cornerExtraction(area_t &area);
    void lineFitting(area_t &area);
    void searchForConnections(area_t &areaFirst, area_t &areaSecond);

private slots:
    void angleTwoVectors(QPointF &first, QPointF &second, QPointF &middle, qreal &angle);
    void angleTwoVectors(QPointF &first, QPointF &second, QPointF &middle, qreal &angle, qreal &vl, qreal &ul);
    void vectorLength(QPointF &first, QPointF &second, qreal &len);
    void resizeCornerArray(int addSize = 50);
    void lineExtract(int indexStart, int indexEnd, area_t &area, qreal &error);
    void getNextPossibleLine(int &indexStart, int &indexEnd, area_t &area);
    void lineEliminateError(int indexStart, int indexEnd, qreal fits, area_t &area);
    void searchForMoreAcurateLine(int indexStart, int indexEnd, area_t &area, qreal &r2, qreal &error, int &actualEnd);
    void getLineParams(int indexStart, int indexEnd, area_t &area, qreal &theta, qreal &ro);
    void getLineParams(int indexStart, int indexEnd, area_t &area, qreal &m, qreal &b, qreal &sErrorY);
    void calculateSquareError(area_t &area, int indexStart, int indexEnd, qreal &ro, qreal &theta, qreal &error);
    void calculateSquareError(area_t &area, int indexStart, int indexEnd, qreal &m, qreal &b, qreal &sErrorY, qreal &error, qreal &r2);
    void addNewLine(int indexStart, int indexEnd, qreal m, qreal b, qreal error, qreal fitsR2, area_t &area);
    void editExistingLine(int indexStart, int indexEnd, qreal m, qreal b, qreal error, qreal fitsR2, area_t &area, int lineIndex);

    qreal getPointsEvaluation(qreal &x1, qreal &y1, qreal &x2, qreal &y2, qreal &sizeHalf);

private:
    QPointF *corners;
    int arrayIndex;
    int cornerCount;
    qreal maxAngle; /**< Maximální úhel kdy se jedná o roh v radiánech. */
    qreal minAngle; /**< Minimální úhel kdy se jedná o roh v radiánech. */
    int linePointStep; /**< Kolik bodů má přímka na začátku. */
    int lineCount;
    int lineArrayCount;
};
#endif // LANDMARKEXTRACTION_H
