#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QTimer>
#include <QModelIndex>
#include <QTableView>

#include "datastructures.h"
#include "datareader.h"
#include "datapreprocessor.h"
#include "pointsetregistration.h"
#include "landmarkextraction.h"
#include "statistics.h"
#include "mapclass.h"
#include "mapprocess.h"
#include "udpclient.h"
#include "submappostproc.h"

namespace Ui {
class MainWindow;
}

/**
 * @brief Hlavní okno aplikace, zde se pracuje se všemi třídami.
 */
class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

    enum MainMenuTabs {
        File,
        StaticRender,
        RealTimeRender,
        ProcessSubmap,
        Settings
    };

    enum RightColumnTabs {
        StatisticsTab,
        SubmapsTab,
        LidarTab,
        LoadedTab
    };

    enum StackedWidget {
        ActualScan,
        Map,
        LidarSettings
    };

    enum StackedWidgetLidar {
        NotConnected,
        Connected,
        Actions
    };

private slots:
    void dataLoaded();
    void reloadStatistics();
    void reloadSubmapTab();
    void nextScan();
    void registrationStepCompleted();
    void correctionStepCompleted();
    void checkRegistrationResults();
    void landmarkExtractionRun();
    void renderStatistics();
    void updateMapRender();
    void zoomIn();
    void zoomOut();
    void onSubmapClicked(QModelIndex index);
    void checkMainMenuEnabled();

    void allReset();
    void changeEnableStaticControls(bool enabled);
    void changeEnabledSubmapControls(bool enabled);
    void clientChangeState();
    void submapLoaded();
    void reloadLoadedTable();
    void reloadLidarTable();

    void on_toolButtonOpenFile_clicked();
    void on_toolButton_clicked();
    void on_toolButtonClose_clicked();
    void on_toolButtonNext_clicked();
    void on_toolButtonStart_clicked();
    void on_toolButtonStop_clicked();
    void on_toolButtonRegistration_clicked();
    void on_toolButtonShowLandmark_clicked();
    void on_toolButtonActualScan_clicked();
    void on_toolButtonMap_clicked();
    void on_toolButtonLidarSettings_clicked();
    void on_pushButtonConnect_clicked();
    void on_pushButtonConnectLidar_clicked();
    void on_pushButtonGrab_clicked();
    void on_pushButtonStop_clicked();
    void on_pushButtonStateInfo_clicked();
    void on_toolButtonRemoveSubmap_clicked();
    void on_toolButtonSaveSubmap_clicked();
    void on_toolButtonLoadSubmap_clicked();
    void on_toolButtonRealTimeStart_clicked();
    void on_toolButtonRealTimeStop_clicked();
    void on_tableViewLoadedSubmaps_clicked(const QModelIndex &index);

private:
    Ui::MainWindow *ui;
    DataReader *dataReader;
    DataPreProcessor *dataPreProcessor;
    LandmarkExtraction *landmarkExtraction;
    Statistics *statistics;
    Statistics *statisticsMap;
    Statistics *statisticsMapRender;
    int activeStaticRender;
    bool renderSubmap;
    int renderedSubmap;
    area_t *areaChanged;
    robotPosition_t *robotPos;
    UdpClient *client;
    SubmapPostProc *postProc;

    MapClass *map;
    MapClass *mapRender;
    MapProcess *mapProcess;

    // registration step
    int areaIndex;
    int connectedIndex;
    int connectedWith;
    int correctionCount;
    bool correctionRunning;
    QVector<int> forbiddenIndexes;

    bool activated;
    bool applyRegistration;
    bool showLandmark;
    bool completedFirstThreat;
    bool completedSecondThreat;
    bool obDraw;

    QTimer *timer;

    void setupTableView(QTableView *tableView);
    void reloadStaticRender();
    void registrationStep();
    void correctionStep(int what, int with);
    void mapCorrectionStep(int start);
    void enableProgressBar();
    void disableProgressBar();

};

#endif // MAINWINDOW_H
