#ifndef MAPCLASS_H
#define MAPCLASS_H

#include <QObject>
#include <armadillo>

#include "datastructures.h"
#include "statistics.h"
#include "submapstruct.h"

using namespace arma;

/**
 * @brief Třída pro práci s mapou a submapami.
 * Obsahuje všechny submapy spolu s pozicemi důležitých objektů a robota.
 */
class MapClass : public QObject
{
    Q_OBJECT

signals:
    void updated();
    void submapChanged();

public:
    MapClass(area_t **allAreas, Statistics *statistics, bool main);
    void setAllAreas(area_t **allAreas);
    int addAreaToActualSubmap(int areaIndex);
    void createNewSubmap();
    void setMaxDistance(int maxD);
    void removeSubmap(int submap);

    QPointF* getSubmatPoints(int submap);
    int getPointsCount(int submap) const;
    /**
     * @brief Vrací počet všech submap.
     * @return Počet všech submap.
     */
    int getSubmapsCount() const { return this->submapsCount; }
    int getRowCountInSubmap(int submap) const;
    int checkRobotPosition();
    /**
     * @brief Vrací směrník na požadovanou submapu.
     * @param i - Index submapy.
     * @return Směrník na danou submapu.
     */
    submap_t* getSubmap(int i) const { return this->submaps[i]; }
    /**
     * @brief Vrací pozice robota v dané submapě.
     * @param i - Index submapy.
     * @return Směrník na pozice robota v dane submapě.
     */
    robotPosition_t* getPosition(int i) const { return this->submaps[i]->position; }

private:
    int submapsCount;
    int actualSubmap;
    int maxDistance;
    submap_t **submaps;

    QPointF *actualSubmatPoints;
    int submatPointsCount;

    area_t **allAreas;
    Statistics *statistics;

    bool main;

    void addLandmarksToSubmap(int areaIndex);
};

#endif // MAPCLASS_H
