#include "mapprocess.h"

#include <qmath.h>
#include <QDebug>

MapProcess::MapProcess(QObject *parent) : QObject(parent)
{
    this->sectorsCount = 0;
    this->actualIndex = -1;
}

MapProcess::~MapProcess()
{
    for(int i = 0; i < this->sectorsCount; i++) {
        for(int h = 0; h < this->sectors[i]->squaresCount; h++) {
            delete this->sectors[i]->squares[h];
        }
        delete this->sectors[i];
    }
}

/**
 * @brief Přidávání submapy do této třídy.
 * @param submap - Přidávaná submapa.
 */
void MapProcess::addSubmap(submap_t *submap)
{
    /// TODO: all reset

    int actualY = -1000000000;
    int actual = 0;
    int y;

    for(uword i = 0; i < submap->allPoints.n_rows; i++) {
        y = submap->allPoints.at(i, 1);
        if(actualY <= y && actualY + 100 > y) {
            // insert
        } else {
            actual = this->getSectorIndex(y, actual);
            actualY = this->sectors[actual]->yCoordStart;
            // insert
        }
    }


    qDebug() << "******* ADD SUBMAP *******" << y;
    for(int i = 0; i < this->sectorsCount; i++) {
        if(this->sectors[i]->assigned) {
            qDebug() << "i: " << i << ", yCoordStart: " << this->sectors[i]->yCoordStart;
        }
    }
}

/**
 * @brief Vrací index sektoru na základě souřadnice y.
 * @param y - Souřadnice y.
 * @param actual - Aktualní index sektoru. Prvně se zkontroluje jestli se nejedná o následující sektor pro zrychlení.
 * @return Index sektoru pro souřadnici y.
 */
int MapProcess::getSectorIndex(int y, int actual)
{
    int yCStart = y >= 0 ? y/100 : y/100 - 1;

    if(sectorsCount == 0 || sectorsCount - actualIndex <= 1)
        this->expandSectorArray();
    if(actualIndex == -1) {
        this->actualIndex = 0;
        this->sectors[this->actualIndex]->yCoordStart = yCStart * 100;
        this->sectors[this->actualIndex]->assigned = true;
        return this->actualIndex;
    } else {
        // check next sector first
        if(this->sectors[actual+1]->assigned) {
            if(this->sectors[actual+1]->yCoordStart <= y && this->sectors[actual+1]->yCoordStart + 100 > y) {
                return actual + 1;
            }
        }

        for(int i = 0; i < this->sectorsCount; i++) {
            if(sectors[i]->yCoordStart <= y && sectors[i]->yCoordStart + 100 > y) {
                return i;
            }
            if(sectors[i]->yCoordStart > y) {
                return this->insertSectorBefore(i, yCStart * 100);
            }
        }
    }
    this->actualIndex++;
    this->sectors[this->actualIndex]->yCoordStart = yCStart * 100;
    //qDebug() << this->sectors[this->actualIndex]->yCoordStart << ", index: " << this->actualIndex;
    this->sectors[this->actualIndex]->assigned = true;
    return this->actualIndex;
}

/**
 * @brief Funkce pro přidávání sektorů doprostřed pole.
 * @param before - Index sektrou, před který se má vložit nový sektor.
 * @param yStart - Souřadnice y, kde začíná vkládaný sektor.
 * @return Vrací index sektoru.
 */
int MapProcess::insertSectorBefore(int before, int yStart)
{
    int h = 0;
    int oldIndex = 0;

    sector_t **old = new sector_t*[this->sectorsCount];
    for(int i = 0; i < this->sectorsCount; i++) {
        old[i] = this->sectors[i];
    }

    this->sectors = new sector_t*[this->sectorsCount];
    for(int i = 0; i < this->sectorsCount; i++) {
        this->sectors[i] = new sector_t();
    }

    while(h < before) {
        this->sectors[h] = old[oldIndex];
        h++;
        oldIndex++;
    }
    this->sectors[before]->yCoordStart = yStart;
    this->sectors[before]->assigned = true;
    h++;
    while(oldIndex < this->sectorsCount) {
        this->sectors[h] = old[oldIndex];
        h++;
        oldIndex++;
    }

    this->actualIndex++;
    return before;
}

/**
 * @brief Zvětšování pole sektorů o velikost 10.
 */
void MapProcess::expandSectorArray()
{
    sector_t **old = new sector_t*[this->sectorsCount];
    for(int i = 0; i < this->sectorsCount; i++) {
        old[i] = this->sectors[i];
    }

    int newCount = this->sectorsCount + 10;
    this->sectors = new sector_t*[newCount];
    for(int i = 0; i < this->sectorsCount; i++) {
        this->sectors[i] = old[i];
    }
    for(int i = this->sectorsCount; i < newCount; i++) {
        this->sectors[i] = new sector_t();
    }
    this->sectorsCount = newCount;
}

/**
 * @brief Vkládaní square_t do sektoru na dané souřadnice.
 * @param sector - Vkládaný sektor.
 * @param x - Souřadnice x (v milimetrech).
 * @param y - Souřadnice y (v milimetrech).
 */
void MapProcess::insertSquare(sector_t *sector, int x, int y)
{
    int xStart = x >= 0 ? x/100 : x/100 - 1;

    if(sector->squaresCount == 0 || sector->squaresCount - sector->actualIndex <= 1) {
        this->expandSquareArray(sector);
    }
    if(sector->actualIndex == -1) {
        sector->actualIndex = 0;
        sector->squares[sector->actualIndex]->assigned = true;
        sector->squares[sector->actualIndex]->xCoordStart = xStart * 100;
        sector->squares[sector->actualIndex]->realRelativeEnd = QPoint(x - xStart, y);
        sector->squares[sector->actualIndex]->points(x - xStart, y) += 1;
    } else {
        for(int i = 0; i < sector->squaresCount; i++) {
            if(sector->squares[i]->xCoordStart <= x && sector->squares[i]->xCoordStart + 100 > x) {
                sector->squares[i]->points(x - xStart, y) += 1;
                if(sector->squares[i]->realRelativeEnd.x() < x - xStart) {
                    sector->squares[i]->realRelativeEnd.setX(x - xStart);
                } else if(sector->squares[i]->realRelativeEnd.y() < y) {
                    sector->squares[i]->realRelativeEnd.setY(y);
                }
            }
            if(sector->squares[i]->xCoordStart > x) {
                /// TODO: insert before
            }
        }
    }
}

/**
 * @brief Realokace pole square_t v daném sektoru o velikost 10.
 * @param sector - Sektor, v kterém se má provést realokace.
 */
void MapProcess::expandSquareArray(sector_t *sector)
{
    square_t **old = new square_t*[sector->squaresCount];
    for(int i = 0; i < sector->squaresCount; i++) {
        old[i] = sector->squares[i];
    }

    int newCount = sector->squaresCount + 10;
    sector->squares = new square_t*[newCount];
    for(int i = 0; i < sector->squaresCount; i++) {
        sector->squares[i] = old[i];
    }
    for(int i = sector->squaresCount; i < newCount; i++) {
        sector->squares[i] = new square_t();
    }
    sector->squaresCount = newCount;
}

