#ifndef MAPPROCESS_H
#define MAPPROCESS_H

#include <QObject>
#include <QPoint>
#include <armadillo>

#include "mapclass.h"

using namespace arma;

/**
 * @brief Struktura představující osu x v každém sektoru.
 *
 * Pro více info viz. třídu MapProcess.
 */
struct square_t {
    square_t() {
        points = zeros<mat>(100, 100);
        xCoordStart = 0;
        realRelativeStart.setX(0);
        realRelativeStart.setY(0);
        realRelativeEnd.setX(0);
        realRelativeEnd.setY(0);
        assigned = false;
    }

    bool assigned;
    int xCoordStart;
    mat points;

    //squere_t *left;
    //squere_t *right;
    square_t *top;
    square_t *bottom;

    QPoint realRelativeStart;
    QPoint realRelativeEnd;
};

/**
 * @brief Struktura představující sektory na ose y.
 *
 * Pro více info viz. třídu MapProcess.
 */
struct sector_t {
    sector_t() {
        assigned = false;
        yCoordStart = 0;
        squaresCount = 0;
        actualIndex = -1;
    }

    bool assigned;
    int squaresCount;
    int actualIndex;
    square_t **squares; // squeres on x axe
    int yCoordStart; // size is always 100 -> yCoordStart + 100 = end
};

/**
 * @brief Třída pro práci s mapou. Zatím testovací a nepoužita.
 *
 * ZATÍM NEPOUŽITA A NEDOKONČENA.
 *
 * Myšlenka byla taková, že mapa kvůli zrychlení prohledávání a zpracování bude rozdělena na částí tzv. sektory (sector_t). Sektory reprezentují osu y, kde každý sektor zabírá 100 jednotek (milimetrů). Každý sektor dále obsahuje dynamické pole squares (square_t). Toto pole představuje osu x, která je opět rozdělena po 100 jednotkách (milimetry). Struktura square_t obsahuje matici 100x100, což symbolizuje body (tam kde je hodnota > 0). Také je to uděláno tak, že nejsou úplně všechny sektory alokovány, pouze ty které se používají. K orientaci pak slouží v square_t proměnné top a bottom, aby se vědělo, jaké oblasti jsou pod a jaké nad daným čtvercem.
 *
 * Takže výsledek by byl takový, že pokud by bylo potřeba prohledat dané místo tak by šlo lehce spočítat o který index sektoru a o který index čtverce se jedná (nemuselo by se prohledávat celou mapu). Další výhodou je, že hodnoty bodů by představovaly počet měření (bodů), které byly v tomto místě naměřené, takže by se dalo brát například pouze body, které mají vyšší hodnotu výskytu (filtrace).
 */
class MapProcess : QObject
{
    Q_OBJECT

public:
    MapProcess(QObject *parent = 0);
    ~MapProcess();
    void addSubmap(submap_t *submap);

private:
    int sectorsCount;
    int actualIndex;
    sector_t **sectors;

    int getSectorIndex(int y, int actual);
    int insertSectorBefore(int before, int yStart);
    void expandSectorArray();
    void insertSquare(sector_t *sector, int x, int y);
    void expandSquareArray(sector_t *sector);
};

#endif // MAPPROCESS_H
