#ifndef POINTSETREGISTRATION_H
#define POINTSETREGISTRATION_H

#include <QObject>
#include <armadillo>

#include "datastructures.h"

using namespace arma;

/**
 * @brief Třída provádějící zarovnávání mračen bodů.
 *
 * Implementovaná metoda Souvislého posunu bodů.
 */
class PointSetRegistration : public QObject
{
    Q_OBJECT

public:
    PointSetRegistration();
    void setAreas(area_t *first, area_t *second);
    void setMatrices(mat &first, mat &second);
    void start(area_t &result);
    void start(mat &result);

private:
    mat rotationMatrixR;
    mat corespondenceMatrixP;
    mat firstSetOfPoints;
    mat secondSetOfPoints;
    vec translationT;
    double sigma;
    area_t *first;
    area_t *second;

    normalization_t normalize(mat &first, mat &second);

    void inicialization(area_t *areaFirst, area_t *areaSecond);
    void inicialization(mat &first, mat &second);
    void denormalize(mat &result, const normalization_t &normal) const;
    void startAlgorithm(mat &T);

    double scale(const mat &mat);
    double vectorSize(const vec &vec1, const vec &vec2);
    double find_P(const mat &X, const mat &Y, double sigma2, float outliers, vec &P1, vec &Pt1, mat &PX);

};
#endif // POINTSETREGISTRATION_H
