#ifndef RENDERAREA_H
#define RENDERAREA_H

#include <QWidget>
#include <QPen>
#include <QPointF>

#include "datastructures.h"

/**
 * @brief Vykreslovací oblast. Zde se vykreslují všechny výsledky, jak už mapy nebo samostatné oblasti.
 */
class RenderArea : public QWidget
{
    Q_OBJECT

signals:
    void updated();
    void statistisUpdate();

public slots:
    void setPen(const QPen &pen);
    void setBrush(const QBrush &brush);
    void setAntialiased(bool antialiased);
    void setTransformed(bool transformed);
    void setShowCorners(bool show);
    void setShowLines(bool show);
    void setShowIntersections(bool show);
    void zoomIn();
    void zoomOut();

public:
    RenderArea(QWidget *parent = 0);

    void setData(area_t *area, bool reset);
    void setData(area_t *area, area_t *area_sec, bool reset);
    void setData(QPointF *points, int pointCount, bool reset);
    void setLandmarks(arma::mat landmarkPositions, landmark_t **land, int count);
    void setRobotPosition(robotPosition_t *position);
    qreal getZoom();
    QPointF getTranslated();

protected:
    void paintEvent(QPaintEvent *event) Q_DECL_OVERRIDE;
    void paintNormal();
    void resetSettings();
    void keyPressEvent(QKeyEvent *event);
    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *);

private:
    QPen pen;
    QPen penSecond;
    QPen robotPen;
    QPen cornerPen;
    QPen linePen;
    QBrush brush;
    bool antialiased;
    bool transformed;
    bool mousePressed;
    bool showCorners;
    bool showLines;
    bool showIntersections;
    QPoint mousePlace;
    QPointF translateArea;
    QPixmap pixmap;
    area_t *area;
    area_t *area_sec;
    qreal zoom;
    robotPosition_t *position;
    int landmarkCount;
    QPointF *landmarks;
    qreal *circle;

};

#endif // RENDERAREA_H
