#include "statistics.h"

/**
 * @brief Nastavení defaultních hodnot.
 */
Statistics::Statistics()
{
    this->m_areaCount = 0;
    this->m_averQuality = 0;
    this->m_translateX = 0;
    this->m_translateY = 0;
    this->m_staticIndex = 0;
    this->m_areaIndex = 0;
    this->m_nextStaticIndex = 0;
    this->m_nextAreaIndex = 0;
    this->m_zoom = 0;
    this->m_sigma = 0;
    this->m_preSigma = 0;
    this->m_couldTransform = false;
}

/**
 * @brief Nastavení počtu oblastí.
 * @param area - Počet oblastí.
 */
void Statistics::setAreaCount(int area)
{
    this->m_areaCount = area;
    emit this->statisticsChanged();
}

/**
 * @brief Nastavení průměrné kvality.
 * @param averQuality - Průměrná kvalita.
 */
void Statistics::setAverQuality(int averQuality)
{
    this->m_averQuality = averQuality;
    emit this->statisticsChanged();
}

/**
 * @brief Nastavení posunutí vykreslovací plochy.
 * @param translateX - Posunutí v X.
 */
void Statistics::setTranslateX(int translateX)
{
    this->m_translateX = translateX;
    emit this->statisticsChanged();
}

/**
 * @brief Nastavení posunutí vykreslovací plochy.
 * @param translateY - Posunutí v Y.
 */
void Statistics::setTranslateY(int translateY)
{
    this->m_translateY = translateY;
    emit this->statisticsChanged();
}

/**
 * @brief Nastavení posunutí vykreslovací plochy.
 * @param translate - Posunutí v X a Y.
 */
void Statistics::setTranslate(QPointF translate)
{
    this->setTranslateX(translate.x());
    this->setTranslateY(translate.y());
}

/**
 * @brief Nastavení indexů registrace.
 * @param staticIndex - Index prvního mračna bodů.
 */
void Statistics::setStaticIndex(int staticIndex)
{
    this->m_staticIndex = m_nextStaticIndex;
    this->m_nextStaticIndex = staticIndex;
    emit this->statisticsChanged();
}

/**
 * @brief Nastavení indexů registrace.
 * @param areaIndex - Index druhého (pohyblivého) mračna bodů.
 */
void Statistics::setAreaIndex(int areaIndex)
{
    this->m_areaIndex = this->m_nextAreaIndex;
    this->m_nextAreaIndex = areaIndex;
    emit this->statisticsChanged();
}

/**
 * @brief Nastavení indexů registrace.
 * @param staticIndex - Index prvního mračna bodů.
 * @param areaIndex - Index druhého (pohyblivého) mračna bodů.
 */
void Statistics::setIndexes(int staticIndex, int areaIndex)
{
    this->setStaticIndex(staticIndex);
    this->setAreaIndex(areaIndex);
}

/**
 * @brief Nastavení počtu submap.
 * @param submaps - Počet submap.
 */
void Statistics::setSubmapsCount(int submaps)
{
    this->m_submapsCount = submaps;
    emit this->statisticsChanged();
}

/**
 * @brief Nastavení přiblížení.
 * @param zoom - Přiblížení.
 */
void Statistics::setZoom(qreal zoom)
{
    this->m_zoom = zoom;
    emit this->statisticsChanged();
}

/**
 * @brief Nastavení rozptylu sigma2.
 * @param sigma - Aktuální rozptyl.
 */
void Statistics::setSigma(qreal sigma)
{
    this->m_preSigma = this->m_sigma;
    this->m_sigma = sigma;
    emit this->statisticsChanged();
}

/**
 * @brief Nastavení úspěšnosti transformace.
 * @param transform - Jestli se povedlo oblasti zarovnat.
 */
void Statistics::setCouldTransform(bool transform)
{
    this->m_couldTransform = transform;
    emit this->statisticsChanged();
}



