#ifndef STATISTICS_H
#define STATISTICS_H

#include <QObject>
#include <QPointF>

/**
 * @brief Třída zajišťující real time statistiky (pravé menu první záložka).
 *
 * Při každé změně se emituje signál statisticsChanged(), který je v hlavním okně odchytávan do SLOTU a statistiky jsou aktualizovány.
 */
class Statistics : public QObject
{
    Q_OBJECT

public:
    Statistics();

    int areaCount() const { return m_areaCount; }
    int averQuality() const { return m_averQuality; }
    int translateX() const { return m_translateX; }
    int translateY() const { return m_translateY; }
    int staticIndex() const { return m_staticIndex; }
    int areaIndex() const { return m_areaIndex; }
    int nextStaticIndex() const { return m_nextStaticIndex; }
    int nextAreaIndex() const { return m_nextAreaIndex; }
    int submapsCount() const { return m_submapsCount; }
    qreal zoom() const { return m_zoom; }
    qreal sigma() const { return m_sigma; }
    qreal preSigma() const { return m_preSigma; }
    bool couldTransform() const { return m_couldTransform; }

public slots:
    void setAreaCount(int area);
    void setAverQuality(int averQuality);
    void setTranslateX(int translateX);
    void setTranslateY(int translateY);
    void setTranslate(QPointF translate);
    void setStaticIndex(int staticIndex);
    void setAreaIndex(int areaIndex);
    void setIndexes(int staticIndex, int areaIndex);
    void setSubmapsCount(int submaps);
    void setZoom(qreal zoom);
    void setSigma(qreal sigma);
    void setCouldTransform(bool transform);

signals:
    void statisticsChanged();

private:
    int m_areaCount;
    int m_averQuality;
    int m_translateX;
    int m_translateY;
    int m_staticIndex;
    int m_areaIndex;
    int m_nextStaticIndex;
    int m_nextAreaIndex;
    int m_submapsCount;
    qreal m_zoom;
    qreal m_sigma;
    qreal m_preSigma;
    bool m_couldTransform;
};

#endif // STATISTICS_H
