#ifndef SUBMAPPOSTPROC_H
#define SUBMAPPOSTPROC_H

#include <QObject>
#include <QFile>

#include "submapstruct.h"

/**
 * @brief Třída pro práci s submapami. Umožňuje ukládání a nahravání ze souboru a odstraňování nepotřebných submap.
 *
 * Po nahrání mapy emituje signál submapLoaded().
 */
class SubmapPostProc : public QObject
{
    Q_OBJECT

signals:
    void submapLoaded();

public:
    SubmapPostProc(QObject *parent = 0);
    void saveSubmap(submap_t *submap, submap_t *withLandmarks);
    void loadSubmap(const QString path);

    submap_t **loadedMaps;
    int submapCount;

private:
    void resizeLoadedMaps();
    void processLine(QByteArray &line, QFile *file);
    void changeState(int state, int count);
    void setPoint(qreal x, qreal y);
    void setRobotPos(qreal x, qreal y);
    void setLandmark(qreal x, qreal y, qreal truth, int type);

    int activeSubmap;
    int pointAdded;
    int positionAdded;
    int actualState;

    /**
     * @brief Aktuální stav při načítání submapy.
     *
     * Pro více info viz. changeState(int state, int count).
     */
    enum State {
        Undefined,
        Points,
        Robot,
        Landmarks
    };
};

#endif // SUBMAPPOSTPROC_H
