#ifndef SUBMAPSTRUCT
#define SUBMAPSTRUCT

#include <armadillo>
#include "datastructures.h"

using namespace arma;

/**
 * @brief Struktura k ukládáná submap.
 */
struct submap_t {
    submap_t() {
        this->firstAreaIndex = -1;
        this->lastAreaIndex = -1;
        this->absoluteRotation = eye<mat>(2, 2);
        this->absoluteTranslation = zeros<vec>(2);
        this->position = new robotPosition_t();
        this->landmarksCount = 100;
        this->landmarksAdded = 0;
        this->landmarksPosition = zeros<mat>(this->landmarksCount, 2);
        this->landmarks = new landmark_t*[this->landmarksCount];
    }

    ~submap_t() {
        delete this->landmarks;
        delete this->position;
    }

    int firstAreaIndex; /**< Index první oblasti area_t přidané do submapy. */
    int lastAreaIndex; /**< Index poslední oblasti area_t přidané do submapy. */
    int landmarksCount; /**< Velikost pole s důležitými objekty. Velikost je větší než aktuální počet objektů, aby se předešlo častým realokacím. */
    int landmarksAdded; /**< Skutečný počet objektů v poli landmark_t. */
    mat absoluteRotation; /**< Součin všech rotací (dosud přidaných do mapy). */
    vec absoluteTranslation; /**< Součet všech translací. */
    mat allPoints; /**< Všechny body submapy, matice n*2, kde n je počet řádku (bodů) a 2 je počet solupců (x, y). */
    robotPosition_t *position; /**< Pozice robota. */
    landmark_t **landmarks; /**< Pole důležitých objektů nalezených v této submapě. */
    mat landmarksPosition; /**< Pozice nalezených důležitých bodů, kde první sloupec je X a druhý Y. */

    void addLandmark(QPointF *point);
};

#endif // SUBMAPSTRUCT

