#ifndef UDPCLIENT_H
#define UDPCLIENT_H

#include <QObject>
#include <QUdpSocket>

#include "rpdev_prot.h"
#include "datastructures.h"
#include "datapreprocessor.h"

/**
 * @brief Struktura pro ukládání modulů dostupných na serveru.
 */
struct modules_t {
    int position;
    QString name;
};

/**
 * @brief Struktura k ukládání nastavení UDP.
 */
struct statisticLidar_t {
    quint16 clientPort;
    quint16 serverPort;
    QHostAddress address;
    QString path;
    bool preprocessData;
    int sample;
    int rate;
};

/**
 * @brief Třída pro připojení se k serveru, ke kterému je připojen LIDAR.
 *
 * Odesílá dva signály. První při změně stavu stateChange() a druhý při změně nastavení UDP statisticChange(), pro real time zobrazení v aplikaci.
 */
class UdpClient : public QObject
{
    Q_OBJECT

public:
    UdpClient(DataPreProcessor *preProcessor, QObject *parent = 0);
    void tryConnect(QString address, quint16 serverPort, quint16 clientPort);
    void connectToLidar(QString path);
    void startGrabbing(int sample, int rate);
    void stopGrabbing();
    void stateMessage();
    int getState() const { return this->state; }
    QString getStateName();
    QStringList getListOfModules() const;
    statisticLidar_t* getStatistic() const { return this->stat; }

    /**
     * @brief Možné stavy vztahu aplikace - server.
     */
    enum ClientState {
        NotConnected,
        ConnectedToServer,
        ConnectedToLidar,
        ReceivingData
    };

signals:
    void stateChange();
    void statisticChange();

private slots:
    void readPendingDatagrams();
    void sendHeartbeat();
    void startPreprocessingData();
    void stopPreprocessingData();

private:
    QUdpSocket *udpSocket;
    modules_t *moduleList;
    QByteArray heartbeatMessage;
    DataPreProcessor *preProcessor;
    statisticLidar_t *stat;

    int modulesCount;
    int state;

    void initSocket();
    void setState(int state);
    void parseModuleList(QByteArray &datagram);
    void parseStateMsg(QByteArray &datagram);
    QByteArray getBytesFromDatagram(int start, int end, QByteArray &datagram);
    int byteArrayToInt(QByteArray array);
    QByteArray convertIntToByteArray(int byteCount, QString bits);
};

#endif // UDPCLIENT_H
